        page    64,132
        title   FUNCPACK - Personal REXX Function Package Example
        comment @

This is FUNCPACK.ASM, a Personal REXX function package example.

Function:

        Install REXX function package, then terminate and stay resident.
        Subsequent calls to the function package are handled.

Entry Conditions:

        Just run FUNCPACK from a DOS prompt - it will install itself.

Exit Conditions:

        This program exits with the return code REXX returned.

Include Files:

        REXXCALL.MAC - maps REXX control block
        CALLREXX.ASM - gets address of REXX interrupt handler
        FINDREXX.ASM - gets REXX interrupt number

Sample Usage:

        To install (from DOS prompt):

        FUNCPACK

        later, from a REXX program:

        say sample()    /* show that the SAMPLE function received control */

To recreate FUNCPACK.COM (the executable version of this program):

        MASM FUNCPACK;
        LINK FUNCPACK;
        EXE2BIN FUNCPACK FUNCPACK.COM

        ERASE FUNCPACK.OBJ
        ERASE FUNCPACK.EXE

This program was tested with Personal REXX 3.00, DOS 5.0, and MASM 6.0.

@
        page
        include rexxcall.mac    ; map REXX control blocks
        page

CR      equ     13
LF      equ     10

;       map entries in table of functions handled by this function package
;       ------------------------------------------------------------------
funcs   struc
fname   db      '        '      ; name of function
min_arg db      ?               ; minimum number of args
max_arg db      ?               ; maximum number of args
handler dw      ?               ; offset of function handler
funcs   ends
func_size equ 12
        page
sample  segment
        assume  cs:sample,ds:sample
        org     100h
main:   jmp     begin

;       table of functions handled by this function package
;       ---------------------------------------------------

table           funcs   <'SAMPLE  ',0,5,hello>
                funcs   <'ECHO    ',1,1,echo>
                funcs   <0H>    ; one byte of zeros marks end of table

;       misc constants and variables
;       ----------------------------

eb              env_block <>    ; environment block
eb_length       equ     $-eb

cb_offset       dw      ?       ; rexxfcn saves call_block addr here
cb_seg          dw      ?       ; ditto

function_name   db      8 dup(?); function name copied from call_block
function_length dw      ?       ; function name length copied from call_block
num_args        dw      ?       ; number of args copied from call_block

my_env_name     db      'SAMPLE   '             ; default environment name

hello_msg       db      'Hello!',CR,LF,'$'      ; "hello" function displays this

return_buffer   db      'return string' ; hard coded return buffer
return_length   equ     $-return_buffer ; goes into ret_count


        page
;       function package entry point
;       ----------------------------
;
;       the following registers are of interest:
;             es:bx = call block
;             es:si = environment block

rexxfcn proc    far
        cld
; at this point we are using the caller's stack
; but our code and data seg registers are already set up.

        push    bp      ; save callers frame pointer

; switch to our own stack

        mov     di,ss   ; so we can save callers stack regs
        mov     bp,sp   ; ...
        cli             ; disable interrupts
        mov     ss,es:[si].env_sseg ; get stack seg
        mov     sp,es:[si].env_sp   ; set stack pointer
        sti             ; allow interrupts
        push    bp      ; save callers sp
        push    di      ; save callers ss
        mov     bp,sp   ; remember position in stack

;       save address of call block
        mov     cb_offset,bx
        mov     dx,es
        mov     cb_seg,dx

;       get number of args
        mov     cx,word ptr es:[bx].arg_count
        mov     num_args,cx

;       get length of function name
        mov     cx,es:[bx].name_length
        mov     function_length,cx

;       copy function name into our storage
        assume  ds:nothing
        mov     dx,word ptr es:[bx].name_addr+2
        mov     ds,dx                   ; ds = seg address of function name
        mov     si,word ptr es:[bx].name_addr+0  ; si = offset
        push    cs
        pop     es                      ; es = cs
        lea     di,function_name        ; es:di = function_name
        rep     movsb                   ; copy function name
        push    cs
        pop     ds                      ; restore ds
        assume  ds:sample               ; tell assembler

;       determine if we can handle this function
        call    which                   ; find out if we can handle it
        or      ax,ax                   ; can we handle it?
        jnz     rexxfcn_done            ; no, let someone else handle it
        page
;       call function handler
        call    [di].handler

;       set up return buffer and return count fields
        mov     di,cb_seg                       ; get address of call block
        mov     es,di
        mov     bx,cb_offset
        mov     cx,return_length                ; get length of result
        mov     word ptr es:[bx].ret_count,cx   ; put length in call block
        lea     di,return_buffer                ; get offset of result
        mov     word ptr es:[bx].ret_addr+0,di  ; put offset into call block
        mov     di,cs                           ; get segment addr of result
        mov     word ptr es:[bx].ret_addr+2,di  ; put segment addr in call block
        mov     ax,0                            ; success
rexxfcn_done:
; restore the caller's stack

        pop     di      ; caller's ss
        pop     si      ; caller's sp
        cli             ; prevent interrupts
        mov     ss,di   ; restore ss
        mov     sp,si   ; restore sp
        sti             ; allow interrupts
        pop     bp      ; restore frame pointer
        ret                                     ; return to REXX
rexxfcn endp
        page
;       decide whether the function is handled in this package
;       ------------------------------------------------------
;
;       on exit,
;       ax=0 if we will handle this function
;            (di = address of table entry for function)
;       ax=1 if it's not one of our functions
;       ax=2 if too many or too few arguments passed

which   proc    near
        mov     ax,1            ; assume not one of our functions
        lea     si,function_name; address of passed function name
        lea     di,table        ; address of table of functions
        mov     cx,function_length      ; length of this function
        or      cx,cx
        jz      which_done
which_loop:
        cmp     byte ptr [di],0 ; end of table?
        je      which_done      ; yes, then not one of our functions
        push    si
        push    di
        repe    cmpsb           ; is it one of our functions?
        pop     di
        pop     si
        je      found           ; yes, process
        mov     cx,function_length ; no, get function length again
        add     di,func_size    ; point to next table entry
        jmp     which_loop      ; check next table entry

;       found a function we will handle
;       -------------------------------

found:                          ; check number of args
        mov     ax,2            ; assume wrong number of args
        mov     cx,num_args     ; get number of args
        cmp     cl,[di].min_arg ; too few args?
        jb      which_done      ; yes, can't handle it
        cmp     cl,[di].max_arg ; too many args?
        ja      which_done      ; yes, can't handle it
        sub     ax,ax           ; no, we will handle this function call

;       only exit from which routine
;       ----------------------------
which_done:
        ret                     ; return to caller
which   endp

        page
;       This is the HELLO function
;       --------------------------

hello   proc    near
        lea     dx,hello_msg    ; address of message to display
        mov     ah,9            ; dos function = display string
        int     21h             ; call DOS to display message

;       return buffer and return length are already set up
        ret                     ; return
hello   endp

;       This is the ECHO function
;       -------------------------

echo    proc    near
        les     bx, dword ptr cb_offset      ; point at call block
        les     bx, es:arg_list_addr[bx]     ; point at argument block
        mov     cx, word ptr es:arg_length[bx]    ; get argument length
        les     bx, es:arg_addr[bx]               ; get argument address
        or      cx, cx                       ; test length
        jz      echo2                        ; quit if nothing there
echo1:
        mov     dl, byte ptr es:[bx]         ; get next character
        mov     ah, 2                        ; function 2
        int     21h                          ; write character
        inc     bx                           ; increment index
        loop    echo1

        mov     dl, CR                       ; write cr/lf
        mov     ah, 2
        int     21h
        mov     dl, LF
        mov     ah, 2
        int     21h
echo2:
        ret
echo    endp

;       end of resident code
;       --------------------
stack   label   word
stacka  db      512 dup('STACK   ')
res_end:
        page
;       code that doesn't stay resident starts here
;       -------------------------------------------
        include callrexx.asm    ; include getint and errint routines
        page

;       install function package, then terminate and stay resident
;       ----------------------------------------------------------

begin   proc    near

;       get address of REXX interrupt handler
;       -------------------------------------
        call    getint          ; get addr of REXX interrupt handler
        jz      gotint          ; if no error, continue
        jmp     errint          ; else print error message and exit
gotint:                         ; continue ...

;       Note: during the rest of this routine BX contains segment address
;       -----------------------------------------------------------------
        mov     bx,cs

;       fill in the environment block
;       -----------------------------

;       zero environment block
        cld                     ; set direction flag
        mov     al,0            ; fill character
        lea     di,eb           ; address of area to zero
        mov     cx,eb_length    ; length of area to zero
        rep     stosb           ; fill environment block with zeros

;       copy environment name to environment block
        mov     cx,8            ; length of environment name
        lea     si,my_env_name  ; address of environment name
        lea     di,eb.env_name  ; where to put it
        rep     movsb           ; copy environment name to env_block

;       set entry point
        lea     si,rexxfcn
        mov     word ptr eb.env_entry_addr+0,si
        mov     word ptr eb.env_entry_addr+2,bx

;       fill in the rest of the environment block
        mov     eb.env_version,1
        mov     eb.env_dseg,bx          ; current stack & data segment
        mov     eb.env_sseg,bx
        lea     si,res_end              ; use this area as as runtime stack
        mov     eb.env_sp,si
        page
;       set up registers, call REXX, and exit
;       -------------------------------------
        mov     ah,REXX_FCN             ; indicate REXX Function Package call
        mov     al,FCN_INSTALL          ; indicate install package request
        mov     bx,cs                   ; get segment address of env_block
        lea     dx,eb                   ; get offset of env_block
        pushf                           ; save flags for REXX's iret
        call    callrexx                ; call REXX interrupt handler
        or      ax,ax                   ; success?
        jnz     error_exit              ; no, exit with REXX return code
        lea     dx,res_end              ; get address of end of resident code
        int     27h                     ; terminate and stay resident

error_exit:                             ; exit with REXX return code
        mov     ah,4Ch                  ; DOS func = exit
        int     21h                     ; error exit - return to DOS
begin   endp

sample  ends
        end  main
