/*

This is RXDUMPV.C, an example of dumping variables in a REXX program.
Illustrates use of Get Next Variable call to Shared Variable Interface.
Coded for Microsoft C version 5.1 small, medium, or large models.

Function:

    Display (dump) active variables.

Entry conditions:

    This program should be run from a REXX program.
    If any parameters are supplied, only those variables with names
    that start with the same characters as the parameters are displayed.

Include Files:

    REXXCALL.H   - maps REXX control block

Calls:

    FINDINT.ASM  - gets address of REXX interrupt handler

Notes:

    To recreate executable version of this program use the command

        MAKE APIMSC                     (small model)
        MAKE MODEL=-AM APIMSC           (medium model)
        MAKE MODEL=-AL APIMSC           (large model)

    RXDUMPV.C uses C preprocessor statements to generate
    the correct code for any of the three models.

    FINDINT.ASM also needs no changes for the different models.

    This program was tested with Personal REXX 2.00, DOS 3.3, and MSC 5.1.

*/

#include <stdio.h>
#include <dos.h>
#include "rexxcall.h"

#ifdef M_I86SM  /* small model */
#define FINDINT findints
#endif
#ifdef M_I86MM  /* medium model */
#define FINDINT findintm
#endif
#ifdef M_I86LM  /* large model */
#define FINDINT findintl
#endif
#define HIGH_WORD(l) ((unsigned) (((unsigned long) (l)) >> 16))
#define LOW_WORD(l) ((unsigned) (((unsigned long) (l)) & 0xffff))

union REGS inregs, outregs;

char norexx_msg[] = "REXX shared variables not available.\n";

main(argc,argv)
int argc;
char *argv[];
{
    unsigned char rexx_interrupt;
    int interrupt_status, i;
    unsigned dataseg;
    struct shvblock sb;
    struct SREGS regs;
    char name[257], value[1025];

    rexx_interrupt = FINDINT(&interrupt_status);
    if (interrupt_status!=0) {
        fprintf(stderr,norexx_msg);
        exit(1);
        }
    inregs.h.ah = REXX_VERSION;
    inregs.h.al = 0;
    int86(rexx_interrupt, &inregs, &outregs);
    if (outregs.h.al == 0) {
        fprintf(stderr,norexx_msg);
        exit(1);
        }

    segread(&regs);
    dataseg = regs.ds;
    memset(&sb,0,sizeof(struct shvblock));
    sb.shvcode = SHVNEXTV;
    sb.shvnam_addr = (FARP) name;
    sb.shvval_addr = (FARP) value;
    sb.shvuser = 256;
    sb.shvbufl = 1024;

    for(;;) {
        inregs.h.ah = REXX_SHV;
#ifdef M_I86LM    /* predefined by microsoft c if large model */
        {
        char far *p = (char far *)&sb;
        inregs.x.bx = FP_SEG(p);
        inregs.x.dx = FP_OFF(p);
        }
#else       /* medium or small model */
        inregs.x.bx = dataseg;
        inregs.x.dx = (unsigned) &sb;
#endif
        int86(rexx_interrupt, &inregs, &outregs);
        if ((int)outregs.x.ax<0) {
            fprintf(stderr,"Error %d from shared variable call.\n",outregs.x.ax);
            exit(2);
            }
        if (sb.shvret & SHVLVAR)
            break;
        if (sb.shvret & ~SHVTRUNC) {
            fprintf(stderr,"Error processing REXX shared variables.\n");
            exit(3);
            }
        name[sb.shvnaml] = 0;
        value[sb.shvvall] = 0;
        if (argc>1) {
            for (i=1; i<argc; i++)
                if (strnicmp(argv[i],name,strlen(argv[i]))==0)
                    break;
            if (i==argc)
                continue;
            }
        printf("Name=%s, Value=%s\n",name,value);
        }
    return(0);
    }
