/*******************************************************************/
/* RXHELP - A general purpose help system. See file RXHELP.DOC     */
/*              for details.                                       */
/* Copyright 1991, Quercus Systems                                 */
/* All Rights Reserved                                             */
/*******************************************************************/

signal on halt
call Init
parse upper arg Misc.HelpFile .
Misc.UnloadRXWINDOW = 0
parse version id n .
if id \= 'REXX/Personal' | n < 2.00 then do
   say 'RXHELP requires Personal REXX version 2.00 or later'
   call Bye
   end

if \fcnpkg('rxwindow') then do              /* load RXWINDOW if necessary */
   'rxwindow /q'
   if \fcnpkg('rxwindow') then do
      say 'Error encountered while loading RXWINDOW function package'
      call Bye
      end
   Misc.UnloadRXWINDOW = 1
   end

if Misc.HelpFile = '' then do
   Misc.HelpFile = 'REXX.HLP'            /* default help file */
   end
else if pos('.',Misc.HelpFile) = 0 then Misc.HelpFile = Misc.HelpFile'.HLP'

if pos(':',Misc.HelpFile) > 0 | pos('\',Misc.HelpFile) > 0 then do
   Misc.HelpFile = dosfname(Misc.HelpFile)
   if dosdir(Misc.HelpFile) = '' then do
      say 'Help file "'Misc.HelpFile'" not found'
      call Bye
      end
   end
else do
   Temp = dospathfind(Misc.HelpFile)
   if Temp = '' then do
      say 'Help file "'Misc.HelpFile'" not found'
      call Bye
      end
   Misc.HelpFile = Temp
   end

parse value parsefn(Misc.HelpFile) with Misc.Drive Misc.Path Misc.FName .
Misc.IndexFile = Misc.Drive':'Misc.Path || Misc.FName'.NDX'
if dosdir(Misc.IndexFile) = '' then do
   say 'Index file "'Misc.IndexFile'" not found'
   call Bye
   end

n = 0
Title = ' ' || linein(Misc.IndexFile) || ' '
do while lines(Misc.IndexFile) > 0
   n = n + 1
   Ind.n = linein(Misc.IndexFile)
   end
call lineout Misc.IndexFile

call PopWindow Row, Col, 0, Title, 1, 0
call Bye

BYE:
   if Misc.UnloadRXWINDOW then do
      'rxunload rxwindow'
      if rc \= 0 then do
         say 'Error encountered while unloading RXWINDOW function package'
         end
      end
   exit
return

INIT:
   Misc.Esc   = '1b'x            /* all significant user keystrokes */
   Misc.Enter = '0d'x
   Misc.Up    = '0048'x
   Misc.Down  = '0050'x
   Misc.PgUp  = '0049'x
   Misc.PgDn  = '0051'x

   Row = 1                       /* upper left corner of the main menu */
   Col = 1

   parse value pcdisplay() with Misc.Display .
   if Misc.Display = 7 then do   /* decide attributes for the windows */
      Misc.Norm = 7              /* monochrome */
      Misc.High = 15
      Misc.Rev = 112
      Misc.Data = 15
      Misc.Rows = 25
      Misc.Cols = 80
      end
   else do
      Misc.Norm = 23             /* color */
      Misc.High = 30
      Misc.Rev = 112
      Misc.Data = 31
      parse value scrsize() with Misc.Rows Misc.Cols
      end
return

/* PopWindow - Handle the setup and display of a window  */
POPWINDOW: procedure expose Misc. Ind.
   parse arg Row, Col, Handle, Title, Index, Type
   /* Row/Col   - Row and col or upper left corner of window */
   /* Handle    - Handle of the previous window (one level back) */
   /* Title     - the heading for the new window to be opened */
   /* Index     - pointer to where we are in the Index array */
   /* Type      - type of window - 0 means menu, 1 means data */

   MaxWidth = length(Title)
   if Type = 1 then do
      call PopData
      end
   else do
      call PopMenu
      /* process user keystrokes */
      call MonitorMenu w,MaxMenu,MaxWidth,Row,Col,Title,Trailer,Index,MaxRow
      call w_close w
      end
return

/* MonitorMenu - process user keystrokes to manipulate a menu */
MONITORMENU: procedure expose Misc. Ind. Menu.
   parse arg Handle,MaxItems,MaxWidth,Row,Col,Header,Trailer,Index,MaxRow
   /* Handle      - the handle for the current window          */
   /* MaxItems    - the # of items on the menu                 */
   /* MaxWidth    - the width of the longest menu item         */
   /* Row/Col     - Row and column of upper left window corner */
   /* Header      - header for window                          */
   /* Trailer     - trailer for window                         */
   /* Index       - pointer to current position in the index   */
   /* MaxRow      - max number of items in window              */

   CurPos = 1
   CurTop = 1
   CurBot = MaxRow
   MinRow = 1
   /* reverse video the initial menu item */
   call w_attr Handle, CurPos+1, 3, MaxWidth, Misc.Rev
   do forever
      Key = inkey()
      select
      when Key = Misc.Down then do
         call w_attr Handle, CurPos+1, 3, MaxWidth, Misc.Norm
         if CurPos < MaxRow
            then CurPos = CurPos + 1
            else CurPos = MinRow
         call w_attr Handle, CurPos+1, 3, MaxWidth, Misc.Rev
         end
      when Key = Misc.Up then do
         call w_attr Handle, CurPos+1, 3, MaxWidth, Misc.Norm
         if CurPos = MinRow
            then CurPos = MaxRow
            else CurPos = CurPos - 1
         call w_attr Handle, CurPos+1, 3, MaxWidth, Misc.Rev
         end
      when Key = Misc.PgUp then do
         if CurTop > 1 then do
            CurTop = max(1,CurTop-MaxRow+1)
            CurBot = CurTop + MaxRow - 1
            call w_clear Handle,,,2,2,MaxRow,MaxWidth+2
            call ShowIt CurTop,CurBot,Handle,Misc.Norm
            CurPos = MinRow
            call w_attr Handle, CurPos+1, 3, MaxWidth, Misc.Rev
            end
         end
      when Key = Misc.PgDn then do
         if CurBot < MaxItems then do
            CurBot = min(MaxItems,CurBot+MaxRow-1)
            CurTop = CurBot - MaxRow + 1
            call w_clear Handle,,,2,2,MaxRow,MaxWidth+2
            call ShowIt CurTop,CurBot,Handle,Misc.Norm
            CurPos = MinRow
            call w_attr Handle, CurPos+1, 3, MaxWidth, Misc.Rev
            end
         end
      when Key = Misc.Enter then do
         /* we want to un-highlight the window that is currently showing */
         /* so that the one about to appear is obviously the "current" one */
         call w_border Handle, , , , , Misc.Norm
         call w_put Handle, 1, 2, Header, length(Header)
         call w_attr Handle, MaxItems+2, 2, length(Trailer), Misc.Norm
         /* pop the next level of menu offset from the current window */
         n = Index
         do i = 1 to CurTop + CurPos - 2
            n = -word(Ind.n,1)
         end
         if word(Ind.n,2) < 0 then do
            Type = 0
            Title = ' ' || substr(Ind.n,wordindex(Ind.n,4)) || ' '
            n = -word(Ind.n,2)
            end
         else do
            Type = 1
            Title = ' ' || substr(Ind.n,wordindex(Ind.n,4)) || ' '
            end
         call PopWindow Row+1, Col+2, Handle, Title, n, Type

         /* the header and trailer lines of the current window need */
         /* to be re-highlighted */
         call w_border Handle,,,,,Misc.High
         call w_put Handle, 1, 2, Header, length(Header), Misc.High
         call w_put Handle, MaxItems+2, 3, Trailer, length(Trailer), Misc.High
         call w_attr Handle, MaxItems+2, 2, length(Trailer), Misc.High
         end
      when Key = Misc.Esc then leave
      otherwise nop
      end  /* select */
      end  /* forever */
return

/* PopData - display a data window */
POPDATA:
   call charin Misc.HelpFile,word(Ind.Index,2),0
   MaxData = word(Ind.index,3)
   do i = 1 to MaxData                      /* get the data */
      Menu.Item.i = substr(linein(Misc.HelpFile), 5)
      l = length(Menu.Item.i) + 2
      if l > MaxWidth
         then MaxWidth = l
      end
   call lineout Misc.HelpFile                  /* close the help file */
   MaxRow = min(MaxData,Misc.Rows-Row-2)
   if MaxRow < MaxData
      then trailer = ' ESC to return, PgUp/PgDn '
      else trailer = ' ESC to return '
   l = length(trailer)
   if l > MaxWidth                          /* we want to make the window */
      then MaxWidth = l                     /* only as wide as necessary */

   /* open this new window */
   w1 = w_open(Row, Col, i + 2, MaxWidth + 4, Misc.Data)
   if w1 = '' then call NoSpace
   call w_border w1, 1, 1, 1, 1, Misc.High
   call w_put w1, 1, 2, Title, , Misc.High
   call w_put w1, MaxRow+2, 3, trailer, length(trailer), Misc.High
   CurTop = 1
   CurBot = MaxRow
   MinRow = 1
   call ShowIt CurTop,CurBot,w1,Misc.Data
   do forever
      Key = inkey()
      select
      when Key = Misc.pgup then do
         if CurTop > 1 then do
            CurTop = max(1,CurTop-MaxRow+1)
            CurBot = CurTop + MaxRow - 1
            call w_clear w1,,,2,2,MaxRow,MaxWidth+2
            call ShowIt CurTop,CurBot,w1,Misc.Data
            end
         end
      when Key = Misc.PgDn then do
         if CurBot < MaxData then do
            CurBot = min(MaxData,CurBot+MaxRow-1)
            CurTop = CurBot - MaxRow + 1
            call w_clear w1,,,2,2,MaxRow,MaxWidth+2
            call ShowIt CurTop,CurBot,w1,Misc.Data
            end
         end
      when Key = Misc.Esc then do
         call w_close w1
         return
         end
      otherwise nop
      end   /* select */
      end   /* do forever */
return

/* PopMenu - display a menu window */
POPMENU:
   n = Index
   MaxMenu = 0
   do forever
      MaxMenu = MaxMenu + 1
      /* get a menu item */
      Menu.Item.MaxMenu = substr(Ind.n, wordindex(Ind.n,4))
      l = length(Menu.Item.MaxMenu)
      if l > MaxWidth
         then MaxWidth = l
      if word(Ind.n,1) = 0
         then leave
      n = -word(Ind.n,1)
   end
   MaxRow = min(MaxMenu,Misc.Rows-Row-2)
   if MaxRow < MaxMenu
      then trailer = ' ESC to return, ENTER to select,  PgUp/PgDn '
      else trailer = ' ESC to return, ENTER to select,  '
   l = length(trailer)
   if l > MaxWidth
      then MaxWidth = l
   w = w_open(Row, Col, MaxRow+3, MaxWidth + 4, Misc.Norm)
   if w = '' then call NoSpace
   call w_border w,,,,,Misc.High
   call w_put w, 1, 2, Title, , Misc.High
   call w_put w, MaxRow+2, 3, trailer, , Misc.High
   call ShowIt 1,MaxRow,w,Misc.Norm
return

NOSPACE:
   'cls'
   say 'RXHELP aborting...'
   say 'Not enough buffer space allocated when RXWINDOW was loaded'
   say 'Try reloading RXWINDOW with the parameter "/s64"'
   call Bye
return

HALT:
   'cls'
   say 'CTRL-Break pressed'
   say 'RXHELP aborting...'
   call Bye
return

/* ShowIt - display the contents of a menu or data screen */
SHOWIT:
   parse arg m,n,w,Attr
   i = 2
   do j = m to n
      call w_put w, i, 3, Menu.Item.j, , Attr
      i = i + 1
      end
return
