/***************************************************************/
/* REXX program to translate BAT files to REXX                 */
/***************************************************************/

parse upper arg BatFileName RexxFileName .

if BatFileName = ''
   then do
      Say 'Syntax is: REXXIFY batchfilename [rexxfilename]'
      exit 1
   end

parse value parsefn(BatFileName) with . . FileName FileExtension
if FileExtension = '-' then BatFileName = BatFileName'.BAT'

if RexxFileName = '' then RexxFileName = FileName'.REX'
parse value parsefn(RexxFileName) with . . FileName FileExtension
if FileExtension = '-' then RexxFileName = RexxFileName'.REX'




/* Read and process the input batch file */
j = 0
shift = 0
errcount = 0
argflag = 0
do while (lines(BatFileName))
    NextLine = Linein(BatFileName)
    call transform NextLine
    end
call Lineout BatFileName

/* generate output */
call dosdel RexxFileName
upper BatFileName
call writeline '/* rexxification of' BatFileName '-' date() '*/'
if argflag then do
    call writeline 'parse arg $argstring'
    call writeline '$shift = 0'
    end
do i=1 to j
    call writeline new.i
    end
if argflag then do
    call writeline 'exit'
    call writeline 'args: procedure expose $argstring $shift'
    call writeline 'return word($argstring,arg(1)+$shift)'
    end
return

/* translate the command */
transform:
line = arg(1)
call parse line
select
    when verb = 'ECHO' then call echo
    when verb = 'FOR' then call for
    when verb = 'GOTO' then call goto
    when verb = 'IF' then call if
    when verb = 'SHIFT' then call shift
    when verb = 'PAUSE' then call pause
    when verb = 'REM' then call rem
    when verb = '"' then call rem
    when verb = 'CD' then call chdir
    when verb = 'CHDIR' then call chdir
    when verb = 'CALL' then call rxcall
    otherwise call command
    end
return

/* parse the given line */
parse:
thing = arg(1)
if substr(thing,1,1) = '@' then do
    parse value thing with '@' thing
    end
k = verify(thing,' /=+"|<>[];,','m')
if k = 0 then do
    verb = thing
    rest = ''
    end
else do
    verb = substr(thing,1,k-1)
    rest = substr(thing,k)
    end
upper verb
return

/* process 'echo' statement */
echo:
w = word(rest,1)
upper w
if w = 'ON' then ans = 'trace a'
else if w = 'OFF' then ans = 'trace n'
else ans = quote(line)
j = j + 1
new.j = ans
return

/* process 'for' statement */
for:
s = quote(line)
do until forrest = ''
   parse var s forfront '%%' forrest
   if forrest \= '' then s = forfront'%'forrest
   end
j = j + 1
new.j = s
drop s forfront forrest
return

/* process the 'goto' statement */
goto:
j = j + 1
l = upper(quote(strip(translate(word(rest,1),' ',':'))))
new.j = 'signal value ' l
return

/* process the 'if' statement */
if:
w = word(rest,1)
upper w
if w = 'NOT' then do
    rest = subword(rest,2)
    part1 = 'if \('
    part2 = ') then'
    end
else do
    part1 = 'if'
    part2 = 'then'
    end
w = word(rest,1)
upper w
select
    when w = 'ERRORLEVEL' then do
        ans = part1 'rc >=' word(rest,2) part2
        cm = subword(rest,3)
        end
    when w = 'EXIST' then do
        ans = part1 'dosdir(' quote(word(rest,2)) ') \= ""' part2
        cm = subword(rest,3)
        end
    when pos('==',rest) \= 0 then do
        k = pos('==',rest)
        rest1 = substr(rest,k+2)
        ans = part1 quote(strip(substr(rest,1,k-1))) '=' quote(word(rest1,1)) ,
            part2
        cm = subword(rest1,2)
        end
    otherwise do
        call error 'Error in IF statement', i
        return
        end
    end
j = j + 1
new.j = ans
call transform cm
return

/* process 'shift' statement */
shift:
j = j + 1
new.j = '$shift = $shift + 1'
shift = 1
return

/* process the 'pause' statement */
pause:
j = j + 1
new.j = quote(line)
return

/* process the 'rem' statement */
rem:
if rest = '' then ans = ''
else ans = quote(line)
j = j + 1
new.j = ans
return

/* process the 'chdir' command */
chdir:
if rest = '' then ans = 'say dosdrive()":"doscd()'
else ans = 'call doschdir' quote(strip(rest))
j = j + 1
new.j = ans
return

/* process the 'call' command - presume external file is still a batch file */
rxcall:
if rest = '' then ans = ''
else ans = quote(strip(rest))
j = j + 1
new.j = ans
return

/* process all other commands */
command:
if substr(verb,1,1) = ':' then do
    ans = substr(word(line,1),2)':'
    if rest \= '' then ans = ans '/*' rest '*/'
    end
else if length(verb)=2 & substr(verb,2,1)=':'
    then ans = 'call dosdrive "'substr(verb,1,1)'"'
else ans = quote(line)
j = j + 1
new.j = ans
return

/* process quotes and %'s in a string */
quote: procedure expose argflag
string = arg(1)
if string == '' then return "''"
inquote = 0
ans = ''
do forever
    i = verify(string,"'%",'m')
    if i = 0 then
        if inquote then return ans||string"'"
        else if string == '' then return ans
        else return ans"'"string"'"
    c = substr(string,i,1)
    if c = '%' & datatype(substr(string,i+1,1),'w') then do
        if i > 1 then
            if inquote then ans = ans||substr(string,1,i-1)
            else ans = ans"'"substr(string,1,i-1)"'"
        if inquote then ans = ans"'"
        ans = ans||'args('substr(string,i+1,1)')'
        string = substr(string,i+2)
        inquote = 0
        argflag = 1
        end
    else if c = "'" then do
        if inquote then ans = ans||substr(string,1,i)"'"
        else ans = ans"'"substr(string,1,i)"'"
        string = substr(string,i+1)
        inquote = 1
        end
    else do
        if inquote then ans = ans||substr(string,1,i)
        else ans = ans"'"substr(string,1,i)
        string = substr(string,i+1)
        inquote = 1
        end
    end

/* issue error messages */
error: procedure expose errcount
if errcount = 0 then do
    call scrwrite 1,1,,2000
    call cursor 1,1
    end
errcount = errcount + 1
say arg(1) 'on line' arg(2)
return

/* write the output line to a disk file, handline errors */
writeline:
parse arg OutputLine
if lineout(RexxFileName, OutputLine) \= 0 then do
    say "Catastrophic Error trying to write File to Disk"
    exit 8
    end
OutputLine = ""
return
